﻿/*
 * Created by SharpDevelop.
 * User: pshai
 * Date: 6.7.2016
 * Time: 10.24
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;

namespace Pshais.Indicators
{
    public class BBandsExtv2 : IIndicatorScript
    {
        int length = 20;
        double devup = 2.0, devdn = 2.0;
        Core.MAType maType = Core.MAType.Sma;

        int deviation = 0;

        bool wentBelow, wentAbove;
        bool resetMid = true;

        bool requireFCC = true; // Full candle cross

        double upper, mid, lower;

        #region IIndicatorScript implementation
        public void Init()
        {

        }

        public IndicatorResult GetResult(IndicatorContext context)
        {
            try
            {
                IPriceInstrument instrument = context.PriceInstrument;
                int position = instrument.Close.Length - (1 + context.Offset + deviation);
                var close = instrument.Close;
                var high = instrument.High;
                var low = instrument.Low;
                var open = instrument.Open;
                var price = close[position];

                var mult = 1.0 / close.Average(); // must use a multiplier for prices since BB don't understand values well if below one (1)...

                int begin, len;
                var upVals = new double[position];
                var mdVals = new double[position];
                var dnVals = new double[position];

                var ret = Core.Bbands(1, position, close.Select(x => x * mult).ToArray(), length, devup, devdn, maType, out begin, out len, upVals, mdVals, dnVals);

                if (ret == Core.RetCode.Success)
                {
                    // Reset values back to normal
                    upVals = upVals.Select(x => x / mult).ToArray();
                    mdVals = mdVals.Select(x => x / mult).ToArray();
                    dnVals = dnVals.Select(x => x / mult).ToArray();

                    upper = upVals[len - 1];
                    mid = mdVals[len - 1];
                    lower = dnVals[len - 1];

                    var result = IndicatorResult.StayNoPosition;

                    // buying
                    if (price < lower)
                    {
                        wentBelow = true;
                        if (!resetMid) wentAbove = false;
                    }
                    if (price > lower && wentBelow)
                    {
                        if (requireFCC)
                        {
                            if (open[position] > lower)
                                result = IndicatorResult.BuyLong;
                        }
                        else
                            result = IndicatorResult.BuyLong;
                    }
                    if (resetMid && price > mid && wentBelow) wentBelow = false;

                    // selling
                    if (price > upper)
                    {
                        if (!resetMid) wentBelow = false;
                        wentAbove = true;
                    }
                    if (price < upper && wentAbove)
                    {
                        if (requireFCC)
                        {
                            if (open[position] < upper)
                                result = IndicatorResult.SellShort;
                        }
                        else
                            result = IndicatorResult.SellShort;
                    }
                    if (resetMid && price < mid && wentAbove) wentAbove = false;


                    return result;
                    /*
                    if (price <= lower)
                        return IndicatorResult.BuyLong;

                    if (price >= upper)
                        return IndicatorResult.SellShort;
                    */
                }
            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex));
            }

            return IndicatorResult.StayNoPosition; // Default output
        }

        public List<ScriptParameter> GetParameters()
        {
            var rtn = new List<ScriptParameter>
            {
                new ScriptParameter("Length", ScriptParameterType.Integer, length.ToString()),
                new ScriptParameter("Dev. Up", ScriptParameterType.Decimal, devup.ToString()),
                new ScriptParameter("Dev. Down", ScriptParameterType.Decimal, devdn.ToString()),
                new ScriptParameter("MA Type", ScriptParameterType.String, maType.ToString()),

                new ScriptParameter("Deviation", ScriptParameterType.Integer, deviation.ToString()),
                new ScriptParameter("Require FCC? (y/n)", ScriptParameterType.String, requireFCC ? "y" : "n"),
                new ScriptParameter("Reset middle? (y/n)", ScriptParameterType.String, resetMid ? "y" : "n"),
            };

            return rtn;
        }

        public void SetParameters(Dictionary<string, object> parameters)
        {
            length = ParseInt(parameters["Length"], length);
            devup = ParseDouble(parameters["Dev. Up"], devup);
            devdn = ParseDouble(parameters["Dev. Down"], devdn);
            maType = ParseMAType(parameters["MA Type"]);
            deviation = ParseInt(parameters["Deviation"], deviation);
            requireFCC = ParseString(parameters["Require FCC? (y/n)"], requireFCC ? "y" : "n") == "y";
            resetMid = ParseString(parameters["Reset middle? (y/n)"], resetMid ? "y" : "n") == "y";
        }

        public List<ChartRecord> GetChartLines()
        {
            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 0, Name = "Upper Band", HexLineColor = "#FF0000" },
                new ChartRecord() { ChartIndex = 0, Name = "MA", HexLineColor = "#aa7700" },
                new ChartRecord() { ChartIndex = 0, Name = "Lower Band", HexLineColor = "#00ff00" }
            };

            return chartLines;
        }

        public List<decimal> GetChartData()
        {
            var chartData = new List<decimal>
            {
                Convert.ToDecimal(upper),
                Convert.ToDecimal(mid),
                Convert.ToDecimal(lower)
            };

            return chartData;
        }

        public string Name
        {
            get
            {
                return "pshai's BBands Extended";
            }
        }

        public int TicksBack
        {
            get
            {
                return Core.BbandsLookback(length, devup, devdn, maType) + 1 + deviation;
            }
        }
        #endregion

        public static Core.MAType ParseMAType(object value)
        {
            Core.MAType val;
            switch (value.ToString().ToLower())
            {
                case "sma":
                    val = Core.MAType.Sma;
                    break;
                case "ema":
                    val = Core.MAType.Ema;
                    break;
                case "dema":
                    val = Core.MAType.Dema;
                    break;
                case "tema":
                    val = Core.MAType.Tema;
                    break;
                case "trima":
                    val = Core.MAType.Trima;
                    break;
                case "t3":
                    val = Core.MAType.T3;
                    break;
                case "mama":
                    val = Core.MAType.Mama;
                    break;
                case "kama":
                    val = Core.MAType.Kama;
                    break;
                case "wma":
                    val = Core.MAType.Wma;
                    break;
                default:
                    val = Core.MAType.Sma;
                    break;
            }

            return val;
        }

        int ParseInt(object value, int lastValue)
        {
            try
            {
                return Convert.ToInt32(value.ToString());
            }
            catch
            {
                return lastValue;
            }
        }

        double ParseDouble(object value, double lastValue)
        {
            var numStyle = System.Globalization.NumberStyles.Any;
            var cult = System.Globalization.CultureInfo.InvariantCulture;

            try
            {
                double output;
                double.TryParse(
                    value.ToString().Replace(',', '.'),
                    numStyle,
                    cult,
                    out output);
                return output;
            }
            catch
            {
                return lastValue;
            }
        }

        string ParseString(object value, string lastValue)
        {
            try
            {
                return Convert.ToString(value);
            }
            catch
            {
                return lastValue;
            }
        }
    }
}